@@ -1,16 +1,15 @@
+import itertools
 import logging
-from typing import Any, Dict, Union, List, Optional
+from typing import Any, Dict, Union, List, Optional, Generator
 from uuid import uuid4
 
-from sqlalchemy import create_engine, Column, Integer, String, DateTime, func, ForeignKey, Boolean, Text
+from sqlalchemy import and_, func, create_engine, Column, Integer, String, DateTime, ForeignKey, Boolean, Text, text
 from sqlalchemy.ext.declarative import declarative_base
 from sqlalchemy.orm import relationship, sessionmaker
-from sqlalchemy.sql import case
+from sqlalchemy.sql import case, null
 
-from haystack.document_store.base import BaseDocumentStore
 from haystack import Document, Label
-from haystack.preprocessor.utils import eval_data_from_file
-
+from haystack.document_store.base import BaseDocumentStore
 
 logger = logging.getLogger(__name__)
 
@@ -42,7 +41,12 @@ class MetaORM(ORMBase):
 
     name = Column(String(100), index=True)
     value = Column(String(1000), index=True)
-    document_id = Column(String(100), ForeignKey("document.id", ondelete="CASCADE", onupdate="CASCADE"), nullable=False)
+    document_id = Column(
+        String(100),
+        ForeignKey("document.id", ondelete="CASCADE", onupdate="CASCADE"),
+        nullable=False,
+        index=True
+    )
 
     documents = relationship(DocumentORM, backref="Meta")
 
@@ -71,6 +75,8 @@ class SQLDocumentStore(BaseDocumentStore):
         update_existing_documents: bool = False,
     ):
         """
+        An SQL backed DocumentStore. Currently supports SQLite, PostgreSQL and MySQL backends.
+
         :param url: URL for SQL database as expected by SQLAlchemy. More info here: https://docs.sqlalchemy.org/en/13/core/engines.html#database-urls
         :param index: The documents are scoped to an index attribute that can be used when writing, querying, or deleting documents. 
                       This parameter sets the default value for document index.
@@ -78,7 +84,8 @@ class SQLDocumentStore(BaseDocumentStore):
         :param update_existing_documents: Whether to update any existing documents with the same ID when adding
                                           documents. When set as True, any document with an existing ID gets updated.
                                           If set to False, an error is raised if the document ID of the document being
-                                          added already exists. Using this parameter coud cause performance degradation for document insertion. 
+                                          added already exists. Using this parameter could cause performance degradation
+                                          for document insertion.
         """
         engine = create_engine(url)
         ORMBase.metadata.create_all(engine)
@@ -96,33 +103,57 @@ class SQLDocumentStore(BaseDocumentStore):
         document = documents[0] if documents else None
         return document
 
-    def get_documents_by_id(self, ids: List[str], index: Optional[str] = None) -> List[Document]:
+    def get_documents_by_id(self, ids: List[str], index: Optional[str] = None, batch_size: int = 10_000) -> List[Document]:
         """Fetch documents by specifying a list of text id strings"""
         index = index or self.index
-        results = self.session.query(DocumentORM).filter(DocumentORM.id.in_(ids), DocumentORM.index == index).all()
-        documents = [self._convert_sql_row_to_document(row) for row in results]
+
+        documents = []
+        for i in range(0, len(ids), batch_size):
+            query = self.session.query(DocumentORM).filter(
+                DocumentORM.id.in_(ids[i: i + batch_size]),
+                DocumentORM.index == index
+            )
+            for row in query.all():
+                documents.append(self._convert_sql_row_to_document(row))
 
         return documents
 
-    def get_documents_by_vector_ids(self, vector_ids: List[str], index: Optional[str] = None):
+    def get_documents_by_vector_ids(self, vector_ids: List[str], index: Optional[str] = None, batch_size: int = 10_000):
         """Fetch documents by specifying a list of text vector id strings"""
         index = index or self.index
-        results = self.session.query(DocumentORM).filter(
-            DocumentORM.vector_id.in_(vector_ids),
-            DocumentORM.index == index
-        ).all()
-        sorted_results = sorted(results, key=lambda doc: vector_ids.index(doc.vector_id))
-        documents = [self._convert_sql_row_to_document(row) for row in sorted_results]
-        return documents
+
+        documents = []
+        for i in range(0, len(vector_ids), batch_size):
+            query = self.session.query(DocumentORM).filter(
+                DocumentORM.vector_id.in_(vector_ids[i: i + batch_size]),
+                DocumentORM.index == index
+            )
+            for row in query.all():
+                documents.append(self._convert_sql_row_to_document(row))
+
+        sorted_documents = sorted(documents, key=lambda doc: vector_ids.index(doc.meta["vector_id"]))  # type: ignore
+        return sorted_documents
 
     def get_all_documents(
-            self,
-            index: Optional[str] = None,
-            filters: Optional[Dict[str, List[str]]] = None,
-            return_embedding: Optional[bool] = None
+        self,
+        index: Optional[str] = None,
+        filters: Optional[Dict[str, List[str]]] = None,
+        return_embedding: Optional[bool] = None,
     ) -> List[Document]:
+        documents = list(self.get_all_documents_generator(index=index, filters=filters))
+        return documents
+
+    def get_all_documents_generator(
+        self,
+        index: Optional[str] = None,
+        filters: Optional[Dict[str, List[str]]] = None,
+        return_embedding: Optional[bool] = None,
+        batch_size: int = 10_000,
+    ) -> Generator[Document, None, None]:
         """
-        Get documents from the document store.
+        Get documents from the document store. Under-the-hood, documents are fetched in batches from the
+        document store and yielded as individual documents. This method can be used to iteratively process
+        a large number of documents without having to load all documents in memory.
 
         :param index: Name of the index to get the documents from. If None, the
                       DocumentStore's default index (self.index) will be used.
@@ -132,68 +163,115 @@ class SQLDocumentStore(BaseDocumentStore):
         """
 
         index = index or self.index
-        query = self.session.query(DocumentORM).filter_by(index=index)
+        # Generally ORM objects kept in memory cause performance issue
+        # Hence using directly column name improve memory and performance.
+        # Refer https://stackoverflow.com/questions/23185319/why-is-loading-sqlalchemy-objects-via-the-orm-5-8x-slower-than-rows-via-a-raw-my
+        documents_query = self.session.query(
+            DocumentORM.id,
+            DocumentORM.text,
+            DocumentORM.vector_id
+        ).filter_by(index=index)
 
         if filters:
-            query = query.join(MetaORM)
+            documents_query = documents_query.join(MetaORM)
             for key, values in filters.items():
-                query = query.filter(MetaORM.name == key, MetaORM.value.in_(values))
+                documents_query = documents_query.filter(
+                    MetaORM.name == key,
+                    MetaORM.value.in_(values),
+                    DocumentORM.id == MetaORM.document_id
+                )
 
-        documents = [self._convert_sql_row_to_document(row) for row in query.all()]
-        return documents
+        documents_map = {}
+        for i, row in enumerate(self._windowed_query(documents_query, DocumentORM.id, batch_size), start=1):
+            documents_map[row.id] = Document(
+                id=row.id,
+                text=row.text,
+                meta=None if row.vector_id is None else {"vector_id": row.vector_id}  # type: ignore
+            )
+            if i % batch_size == 0:
+                documents_map = self._get_documents_meta(documents_map)
+                yield from documents_map.values()
+                documents_map = {}
+        if documents_map:
+            documents_map = self._get_documents_meta(documents_map)
+            yield from documents_map.values()
+
+    def _get_documents_meta(self, documents_map):
+        doc_ids = documents_map.keys()
+        meta_query = self.session.query(
+         MetaORM.document_id,
+         MetaORM.name,
+         MetaORM.value
+        ).filter(MetaORM.document_id.in_(doc_ids))
+
+        for row in meta_query.all():
+            documents_map[row.document_id].meta[row.name] = row.value  # type: ignore
+        return documents_map
 
     def get_all_labels(self, index=None, filters: Optional[dict] = None):
         """
         Return all labels in the document store
         """
         index = index or self.label_index
+        # TODO: Use batch_size
         label_rows = self.session.query(LabelORM).filter_by(index=index).all()
         labels = [self._convert_sql_row_to_label(row) for row in label_rows]
 
         return labels
 
-    def write_documents(self, documents: Union[List[dict], List[Document]], index: Optional[str] = None):
+    def write_documents(
+        self, documents: Union[List[dict], List[Document]], index: Optional[str] = None, batch_size: int = 10_000
+    ):
         """
         Indexes documents for later queries.
 
-      :param documents: a list of Python dictionaries or a list of Haystack Document objects.
+        :param documents: a list of Python dictionaries or a list of Haystack Document objects.
                           For documents as dictionaries, the format is {"text": "<the-actual-text>"}.
                           Optionally: Include meta data via {"text": "<the-actual-text>",
                           "meta":{"name": "<some-document-name>, "author": "somebody", ...}}
                           It can be used for filtering and is accessible in the responses of the Finder.
         :param index: add an optional index attribute to documents. It can be later used for filtering. For instance,
                       documents for evaluation can be indexed in a separate index than the documents for search.
+        :param batch_size: When working with large number of documents, batching can help reduce memory footprint.
 
         :return: None
         """
 
-        # Make sure we comply to Document class format
-        document_objects = [Document.from_dict(d) if isinstance(d, dict) else d for d in documents]
         index = index or self.index
-        for doc in document_objects:
-            meta_fields = doc.meta or {}
-            vector_id = meta_fields.get("vector_id")
-            meta_orms = [MetaORM(name=key, value=value) for key, value in meta_fields.items()]
-            doc_orm = DocumentORM(id=doc.id, text=doc.text, vector_id=vector_id, meta=meta_orms, index=index)
-            if self.update_existing_documents:
-                # First old meta data cleaning is required
-                self.session.query(MetaORM).filter_by(document_id=doc.id).delete()
-                self.session.merge(doc_orm)
-            else:
-                self.session.add(doc_orm)
-        try:
-            self.session.commit()
-        except Exception as ex:
-            logger.error(f"Transaction rollback: {ex.__cause__}")
-            # Rollback is important here otherwise self.session will be in inconsistent state and next call will fail
-            self.session.rollback()
-            raise ex
+        if len(documents) == 0:
+            return
+        # Make sure we comply to Document class format
+        if isinstance(documents[0], dict):
+            document_objects = [Document.from_dict(d) if isinstance(d, dict) else d for d in documents]
+        else:
+            document_objects = documents
+
+        for i in range(0, len(document_objects), batch_size):
+            for doc in document_objects[i: i + batch_size]:
+                meta_fields = doc.meta or {}
+                vector_id = meta_fields.pop("vector_id", None)
+                meta_orms = [MetaORM(name=key, value=value) for key, value in meta_fields.items()]
+                doc_orm = DocumentORM(id=doc.id, text=doc.text, vector_id=vector_id, meta=meta_orms, index=index)
+                if self.update_existing_documents:
+                    # First old meta data cleaning is required
+                    self.session.query(MetaORM).filter_by(document_id=doc.id).delete()
+                    self.session.merge(doc_orm)
+                else:
+                    self.session.add(doc_orm)
+            try:
+                self.session.commit()
+            except Exception as ex:
+                logger.error(f"Transaction rollback: {ex.__cause__}")
+                # Rollback is important here otherwise self.session will be in inconsistent state and next call will fail
+                self.session.rollback()
+                raise ex
 
     def write_labels(self, labels, index=None):
         """Write annotation labels into document store."""
 
         labels = [Label.from_dict(l) if isinstance(l, dict) else l for l in labels]
         index = index or self.label_index
+        # TODO: Use batch_size
         for label in labels:
             label_orm = LabelORM(
                 document_id=label.document_id,
@@ -210,23 +288,38 @@ class SQLDocumentStore(BaseDocumentStore):
             self.session.add(label_orm)
         self.session.commit()
 
-    def update_vector_ids(self, vector_id_map: Dict[str, str], index: Optional[str] = None):
+    def update_vector_ids(self, vector_id_map: Dict[str, str], index: Optional[str] = None, batch_size: int = 10_000):
         """
         Update vector_ids for given document_ids.
 
         :param vector_id_map: dict containing mapping of document_id -> vector_id.
         :param index: filter documents by the optional index attribute for documents in database.
+        :param batch_size: When working with large number of documents, batching can help reduce memory footprint.
         """
         index = index or self.index
-        self.session.query(DocumentORM).filter(
-            DocumentORM.id.in_(vector_id_map),
-            DocumentORM.index == index
-        ).update({
-            DocumentORM.vector_id: case(
-                vector_id_map,
-                value=DocumentORM.id,
-            )
-        }, synchronize_session=False)
+        for chunk_map in self.chunked_dict(vector_id_map, size=batch_size):
+            self.session.query(DocumentORM).filter(
+                DocumentORM.id.in_(chunk_map),
+                DocumentORM.index == index
+            ).update({
+                DocumentORM.vector_id: case(
+                    chunk_map,
+                    value=DocumentORM.id,
+                )
+            }, synchronize_session=False)
+            try:
+                self.session.commit()
+            except Exception as ex:
+                logger.error(f"Transaction rollback: {ex.__cause__}")
+                self.session.rollback()
+                raise ex
+
+    def reset_vector_ids(self, index: Optional[str] = None):
+        """
+        Set vector IDs for all documents as None
+        """
+        index = index or self.index
+        self.session.query(DocumentORM).filter_by(index=index).update({DocumentORM.vector_id: null()})
         self.session.commit()
 
     def update_document_meta(self, id: str, meta: Dict[str, str]):
@@ -239,22 +332,6 @@ class SQLDocumentStore(BaseDocumentStore):
             self.session.add(m)
         self.session.commit()
 
-    def add_eval_data(self, filename: str, doc_index: str = "eval_document", label_index: str = "label"):
-        """
-        Adds a SQuAD-formatted file to the DocumentStore in order to be able to perform evaluation on it.
-
-        :param filename: Name of the file containing evaluation data
-        :type filename: str
-        :param doc_index: Elasticsearch index where evaluation documents should be stored
-        :type doc_index: str
-        :param label_index: Elasticsearch index where labeled questions should be stored
-        :type label_index: str
-        """
-
-        docs, labels = eval_data_from_file(filename)
-        self.write_documents(docs, index=doc_index)
-        self.write_labels(labels, index=label_index)
-
     def get_document_count(self, filters: Optional[Dict[str, List[str]]] = None, index: Optional[str] = None) -> int:
         """
         Return the number of documents in the document store.
@@ -336,3 +413,56 @@ class SQLDocumentStore(BaseDocumentStore):
             session.add(instance)
             session.commit()
             return instance
+
+    def chunked_dict(self, dictionary, size):
+        it = iter(dictionary)
+        for i in range(0, len(dictionary), size):
+            yield {k: dictionary[k] for k in itertools.islice(it, size)}
+
+    def _column_windows(self, session, column, windowsize):
+        """Return a series of WHERE clauses against
+        a given column that break it into windows.
+
+        Result is an iterable of tuples, consisting of
+        ((start, end), whereclause), where (start, end) are the ids.
+
+        The code is taken from: https://github.com/sqlalchemy/sqlalchemy/wiki/RangeQuery-and-WindowedRangeQuery
+        """
+
+        def int_for_range(start_id, end_id):
+            if end_id:
+                return and_(
+                    column >= start_id,
+                    column < end_id
+                )
+            else:
+                return column >= start_id
+
+        q = session.query(
+            column,
+            func.row_number(). \
+                over(order_by=column). \
+                label('rownum')
+        ). \
+            from_self(column)
+        if windowsize > 1:
+            q = q.filter(text("rownum %% %d=1" % windowsize))
+
+        intervals = [id for id, in q]
+
+        while intervals:
+            start = intervals.pop(0)
+            if intervals:
+                end = intervals[0]
+            else:
+                end = None
+            yield int_for_range(start, end)
+
+    def _windowed_query(self, q, column, windowsize):
+        """"Break a Query into windows on a given column."""
+
+        for whereclause in self._column_windows(
+                q.session,
+                column, windowsize):
+            for row in q.filter(whereclause).order_by(column):
+                yield row
