@@ -1,6 +1,9 @@
+import pytest
 
 
-def test_tfidf_retriever(inmemory_document_store, tfidf_retriever):
+@pytest.mark.parametrize("retriever", ["tfidf"], indirect=True)
+@pytest.mark.parametrize("document_store", ["memory"], indirect=True)
+def test_tfidf_retriever(document_store, retriever):
 
     test_docs = [
         {"id": "26f84672c6d7aaeb8e2cd53e9c62d62d", "name": "testing the finder 1", "text": "godzilla says hello"},
@@ -8,10 +11,10 @@ def test_tfidf_retriever(inmemory_document_store, tfidf_retriever):
         {"name": "testing the finder 3", "text": "alien says arghh"}
     ]
 
-    inmemory_document_store.write_documents(test_docs)
+    document_store.write_documents(test_docs)
 
-    tfidf_retriever.fit()
-    doc = tfidf_retriever.retrieve("godzilla", top_k=1)[0]
+    retriever.fit()
+    doc = retriever.retrieve("godzilla", top_k=1)[0]
     assert doc.id == "26f84672c6d7aaeb8e2cd53e9c62d62d"
     assert doc.text == 'godzilla says hello'
     assert doc.meta == {"name": "testing the finder 1"}
